/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QAction>
#include <QCloseEvent>
#include <QDesktopWidget>
#include <QDir>
#include <QFile>
#include <QFileDialog>

#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QGridLayout>
#include <QLayout>
#include <QMenu>
#include <QPixmap>
#include <QProcess>
#include <QPushButton>
#include <QSettings>
#include <QSignalMapper>
#include <QTextStream>
#include <QTextDocumentFragment>
#include <QToolButton>
#include <QMessageBox>

#include <QDebug>

#include "defines.h"
#include "EquationPixmapItem.h"

#include "DialogReplace.h"
#include "DialogPreferences.h"
#include "DialogAbout.h"
#include "EquationTemplateWidget.h"
#include "FrameMathTab.h"
#include "LatexEditor.h"
#include "WidgetFind.h"
#include "WidgetToolMenu.h"


#include "MainWindow.h"
#include "ui_mainwindow.h"


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
    ui(new Ui::MainWindowClass),
    equation(0),
    isDirty(false), zoomFactor(100), nUpdates(0)

{
    ui->setupUi(this);

    setWindowTitle(APP_FULL_NAME);

    scene = new QGraphicsScene();

    ui->View->setScene(scene);
    ui->fgColorPicker->setCurrentIndex(2);
    ui->bgColorPicker->setCurrentIndex(1);


    editor = ui->editEquation;
    findWidget = ui->findWidget;

    templateTab = new FrameMathTab;

    replaceDialog = new DialogReplace(this);
    preferencesDialog = new DialogPreferences(this);
    aboutDialog = new DialogAbout(this);

    equation = new EquationPixmapItem;
    equation->setVisible(true);
    equation->setAcceptDrops(true);
    equation->setToolTip("Drag and Drop this");

    scene->addItem(equation);

    readSettings();

    setActions();
    setSymbolsPanel();
    setMathPanelTabsActions();
    setSignals();

    setupParser();
    loadTexTemplate();

    setCurrentFile("");
    ui->fontSizeComboBox->setCurrentIndex(0);

    findWidget->hide();

    editor->setFocus();

    updateRecentFileActions();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::setActions()
{
    /* Menus for Symbols Toolbar */
    toolDeco = new WidgetToolMenu(2, 1, this);
    toolDeco->addAction(ui->actionDeco_super);
    toolDeco->addAction(ui->actionDeco_under);
    toolDeco->addAction(ui->actionDeco_overset);
    toolDeco->addAction(ui->actionDeco_underset);

    toolFunc= new WidgetToolMenu(2, 1, this);
    toolFunc->addAction(ui->actionFunc_frac);
    toolFunc->addAction(ui->actionFunc_root);
    toolFunc->addAction(ui->actionFunc_fracNoHLine);
    toolFunc->addAction(ui->actionFunc_sqrt);

    toolDelim= new WidgetToolMenu(3, 1, this);
    toolDelim->addAction(ui->actionDelim_paraSimple);
    toolDelim->addAction(ui->actionDelim_squareSimple);
    toolDelim->addAction(ui->actionDelim_ceilSimple);
    toolDelim->addAction(ui->actionDelim_curlySimple);
    toolDelim->addAction(ui->actionDelim_angleSimple);
    toolDelim->addAction(ui->actionDelim_floorSimple);
    toolDelim->addAction(ui->actionDelim_vertSimple);
    toolDelim->addAction(ui->actionDelim_barsSimple);


    toolAccents = new WidgetToolMenu(3, 1, this);
    toolAccents->addAction(ui->actionAccents_hat);
    toolAccents->addAction(ui->actionAccents_acute);
    toolAccents->addAction(ui->actionAccents_breve);
    toolAccents->addAction(ui->actionAccents_tilde);
    toolAccents->addAction(ui->actionAccents_underleftrightarrow);
    toolAccents->addAction(ui->actionAccents_underline);
    toolAccents->addAction(ui->actionAccents_bar);
    toolAccents->addAction(ui->actionAccents_check);
    toolAccents->addAction(ui->actionAccents_dot);
    toolAccents->addAction(ui->actionAccents_dotdouble);
    toolAccents->addAction(ui->actionAccents_grave);
    toolAccents->addAction(ui->actionAccents_hatwide);
    toolAccents->addAction(ui->actionAccents_overbrace);
    toolAccents->addAction(ui->actionAccents_overleftarrow);
    toolAccents->addAction(ui->actionAccents_overleftrightarrow);
    toolAccents->addAction(ui->actionAccents_overrightarrow);
    toolAccents->addAction(ui->actionAccents_overline);
    toolAccents->addAction(ui->actionAccents_tildewide);
    toolAccents->addAction(ui->actionAccents_underbrace);
    toolAccents->addAction(ui->actionAccents_underleftarrow);
    toolAccents->addAction(ui->actionAccents_underrightarrow);
    toolAccents->addAction(ui->actionAccents_vector);

    toolOps = new WidgetToolMenu( 3, 1, this);
    toolOps->addAction(ui->actionOps_pm);
    toolOps->addAction(ui->actionOps_times);
    toolOps->addAction(ui->actionOps_div);
    toolOps->addAction(ui->actionOps_mp);
    toolOps->addAction(ui->actionOps_cap);
    toolOps->addAction(ui->actionOps_cup);
    toolOps->addAction(ui->actionOps_sqcap);
    toolOps->addAction(ui->actionOps_sqcup);
    toolOps->addAction(ui->actionOps_uplus);
    toolOps->addAction(ui->actionOps_amalg);
    toolOps->addAction(ui->actionOps_ominus);
    toolOps->addAction(ui->actionOps_oplus);
    toolOps->addAction(ui->actionOps_otimes);
    toolOps->addAction(ui->actionOps_oslash);
    toolOps->addAction(ui->actionOps_odot);
    toolOps->addAction(ui->actionOps_cdot);
    toolOps->addAction(ui->actionOps_circ);
    toolOps->addAction(ui->actionOps_bullet);
    toolOps->addAction(ui->actionOps_bigcirc);
    toolOps->addAction(ui->actionOps_star);
    toolOps->addAction(ui->actionOps_diamond);
    toolOps->addAction(ui->actionOps_vee);
    toolOps->addAction(ui->actionOps_wedge);
    toolOps->addAction(ui->actionOps_setminus);
    toolOps->addAction(ui->actionOps_wr);
    toolOps->addAction(ui->actionOps_dagger);
    toolOps->addAction(ui->actionOps_daggerd);
    toolOps->addAction(ui->actionOps_triangleleft);
    toolOps->addAction(ui->actionOps_triangleright);
    toolOps->addAction(ui->actionOps_bigtriangleup);
    toolOps->addAction(ui->actionOps_bigtriangledown);

    toolRelations = new WidgetToolMenu( 3, 1, this);
    toolRelations->addAction(ui->actionRel_leq);
    toolRelations->addAction(ui->actionRel_neq);
    toolRelations->addAction(ui->actionRel_simeq);
    toolRelations->addAction(ui->actionRel_geq);
    toolRelations->addAction(ui->actionRel_equiv);
    toolRelations->addAction(ui->actionRel_models);
    toolRelations->addAction(ui->actionRel_prec);
    toolRelations->addAction(ui->actionRel_succ);
    toolRelations->addAction(ui->actionRel_sim);
    toolRelations->addAction(ui->actionRel_perp);
    toolRelations->addAction(ui->actionRel_preceq);
    toolRelations->addAction(ui->actionRel_succeq);
    toolRelations->addAction(ui->actionRel_mid);
    toolRelations->addAction(ui->actionRel_lless);
    toolRelations->addAction(ui->actionRel_ggreater);
    toolRelations->addAction(ui->actionRel_asymp);
    toolRelations->addAction(ui->actionRel_parallel);
    toolRelations->addAction(ui->actionRel_subset);
    toolRelations->addAction(ui->actionRel_supset);
    toolRelations->addAction(ui->actionRel_approx);
    toolRelations->addAction(ui->actionRel_smile);
    toolRelations->addAction(ui->actionRel_subseteq);
    toolRelations->addAction(ui->actionRel_supseteq);
    toolRelations->addAction(ui->actionRel_cong);
    toolRelations->addAction(ui->actionRel_frown);
    toolRelations->addAction(ui->actionRel_sqsubseteq);
    toolRelations->addAction(ui->actionRel_sqsupseteq);
    toolRelations->addAction(ui->actionRel_dotequal);
    toolRelations->addAction(ui->actionRel_in);
    toolRelations->addAction(ui->actionRel_ni);
    toolRelations->addAction(ui->actionRel_propto);
    toolRelations->addAction(ui->actionRel_notin);
    toolRelations->addAction(ui->actionRel_vdash);
    toolRelations->addAction(ui->actionRel_dashv);
    toolRelations->addAction(ui->actionRel_bowtie);

    toolMisc = new WidgetToolMenu(3, 1, this);
    toolMisc->addAction(ui->actionMisc_infty);
    toolMisc->addAction(ui->actionMisc_forall);
    toolMisc->addAction(ui->actionMisc_exists);
    toolMisc->addAction(ui->actionMisc_nabla);
    toolMisc->addAction(ui->actionMisc_partial);
    toolMisc->addAction(ui->actionMisc_prime);
    toolMisc->addAction(ui->actionMisc_ell);
    toolMisc->addAction(ui->actionMisc_emptyset);
    toolMisc->addAction(ui->actionMisc_imath);
    toolMisc->addAction(ui->actionMisc_jmath);
    toolMisc->addAction(ui->actionMisc_wp);
    toolMisc->addAction(ui->actionMisc_textrmO);
    toolMisc->addAction(ui->actionMisc_hbar);

    toolSets = new WidgetToolMenu(3,1, this);
    toolSets->addAction(ui->actionSets_re);
    toolSets->addAction(ui->actionSets_mathbbZ);
    toolSets->addAction(ui->actionSets_mathcalH);
    toolSets->addAction(ui->actionSets_im);
    toolSets->addAction(ui->actionSets_mathbbC);
    toolSets->addAction(ui->actionSets_mathcalF);
    toolSets->addAction(ui->actionSets_aleph);
    toolSets->addAction(ui->actionSets_mathbbH);
    toolSets->addAction(ui->actionSets_mathcalL);
    toolSets->addAction(ui->actionSets_mathbbN);
    toolSets->addAction(ui->actionSets_mathbbQ);
    toolSets->addAction(ui->actionSets_mathcalO);
    toolSets->addAction(ui->actionSets_mathbbR);

    toolGreek = new WidgetToolMenu(3,1 , this);
    toolGreek->addAction(ui->actionGreek_alpha);
    toolGreek->addAction(ui->actionGreek_beta);
    toolGreek->addAction(ui->actionGreek_chi);
    toolGreek->addAction(ui->actionGreek_delta);
    toolGreek->addAction(ui->actionGreek_epsilon);
    toolGreek->addAction(ui->actionGreek_epsilonVar);
    toolGreek->addAction(ui->actionGreek_eta);
    toolGreek->addAction(ui->actionGreek_gamma);
    toolGreek->addAction(ui->actionGreek_iota);
    toolGreek->addAction(ui->actionGreek_kappa);
    toolGreek->addAction(ui->actionGreek_lambda);
    toolGreek->addAction(ui->actionGreek_mu);
    toolGreek->addAction(ui->actionGreek_nu);
    toolGreek->addAction(ui->actionGreek_omega);
    toolGreek->addAction(ui->actionGreek_phi);
    toolGreek->addAction(ui->actionGreek_phiVar);
    toolGreek->addAction(ui->actionGreek_pi);
    toolGreek->addAction(ui->actionGreek_piVar);
    toolGreek->addAction(ui->actionGreek_psi);
    toolGreek->addAction(ui->actionGreek_rho);
    toolGreek->addAction(ui->actionGreek_rhoVar);
    toolGreek->addAction(ui->actionGreek_sigma);
    toolGreek->addAction(ui->actionGreek_sigmaVar);
    toolGreek->addAction(ui->actionGreek_tau);
    toolGreek->addAction(ui->actionGreek_theta);
    toolGreek->addAction(ui->actionGreek_thetaVar);
    toolGreek->addAction(ui->actionGreek_upsilon);
    toolGreek->addAction(ui->actionGreek_xi);
    toolGreek->addAction(ui->actionGreek_zeta);

    toolGreekBig = new WidgetToolMenu(3, 1, this);
    toolGreekBig->addAction(ui->actionGreekBig_delta);
    toolGreekBig->addAction(ui->actionGreekBig_gamma);
    toolGreekBig->addAction(ui->actionGreekBig_lambda);
    toolGreekBig->addAction(ui->actionGreekBig_phi);
    toolGreekBig->addAction(ui->actionGreekBig_pi);
    toolGreekBig->addAction(ui->actionGreekBig_psi);
    toolGreekBig->addAction(ui->actionGreekBig_sigma);
    toolGreekBig->addAction(ui->actionGreekBig_theta);
    toolGreekBig->addAction(ui->actionGreekBig_upsilon);
    toolGreekBig->addAction(ui->actionGreekBig_xi);

    toolSums = new WidgetToolMenu( 3, 1, this);
    toolSums->addAction(ui->actionSum_sum);
    toolSums->addAction(ui->actionSum_sumArgs);
    toolSums->addAction(ui->actionSum_sumArgs2);

    toolIntegrals = new WidgetToolMenu(2,1, this);
    toolIntegrals->addAction(ui->actionInt_simple);
    toolIntegrals->addAction(ui->actionInt_oint);
    toolIntegrals->addAction(ui->actionInt_intDouble);
    toolIntegrals->addAction(ui->actionInt_oint2);
    toolIntegrals->addAction(ui->actionInt_int2);
    toolIntegrals->addAction(ui->actionInt_intTriple);
    toolIntegrals->addAction(ui->actionInt_intQuadruple);
    toolIntegrals->addAction(ui->actionInt_ointop);
    toolIntegrals->addAction(ui->actionInt_dots);

    toolProd = new WidgetToolMenu(2, 1, this);
    toolProd->addAction(ui->actionProd_prod);
    toolProd->addAction(ui->actionProd_coproduct);
    toolProd->addAction(ui->actionProd_prodArgs);
    toolProd->addAction(ui->actionProd_coproductArgs);
    toolProd->addAction(ui->actionProd_prodArgs2);
    toolProd->addAction(ui->actionProd_coproductArgs2);

    toolArrows = new WidgetToolMenu(3, 1, this);
    toolArrows->addAction(ui->actionArrow_rightarrow);
    toolArrows->addAction(ui->actionArrow_leftarrow);
    toolArrows->addAction(ui->actionArrow_downarrow);
    toolArrows->addAction(ui->actionArrow_uparrow);
    toolArrows->addAction(ui->actionArrow_updownarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow2);
    toolArrows->addAction(ui->actionArrow_leftarrow2);
    toolArrows->addAction(ui->actionArrow_rightarrow2);
    toolArrows->addAction(ui->actionArrow_downarrow2);
    toolArrows->addAction(ui->actionArrow_uparrow2);
    toolArrows->addAction(ui->actionArrow_updownarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_longleftarrow);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftharpoondown);
    toolArrows->addAction(ui->actionArrow_rightharpoondown);
    toolArrows->addAction(ui->actionArrow_mapsto);
    toolArrows->addAction(ui->actionArrow_mapsto2);
    toolArrows->addAction(ui->actionArrow_arrowNW);
    toolArrows->addAction(ui->actionArrow_arrowNE);
    toolArrows->addAction(ui->actionArrow_leftharpoonup);
    toolArrows->addAction(ui->actionArrow_rightharpoonup);
    toolArrows->addAction(ui->actionArrow_hookleftarrow);
    toolArrows->addAction(ui->actionArrow_hookrightarrow);
    toolArrows->addAction(ui->actionArrow_arrowSW);
    toolArrows->addAction(ui->actionArrow_arrowSE);
    toolArrows->addAction(ui->actionArrow_rightleftharpoons);

    for (int i = 0; i < MAX_RECENT_FILES; ++i) {
        recentFileActions[i] = new QAction(this);
        recentFileActions[i]->setVisible(false);
        connect(recentFileActions[i], SIGNAL(triggered()),
                this, SLOT(openRecentFile()));
        ui->menu_File->insertAction(ui->actionRecents ,recentFileActions[i]);
    }



}

void MainWindow::setSymbolsPanel()
{

    QFrame *mSymGrid = new QFrame;
    QGridLayout *mSymGridLayout = new QGridLayout;
    mSymGridLayout->setSpacing(0);
    mSymGridLayout->setContentsMargins(0,0,0,0);

    mSymGrid->setLayout(mSymGridLayout);

    // first row
    mSymGridLayout->addWidget(toolDeco, 0, 0);
    mSymGridLayout->addWidget(toolFunc, 0, 1);
    mSymGridLayout->addWidget(toolDelim, 0, 2);
    mSymGridLayout->addWidget(toolAccents, 0, 3);
    mSymGridLayout->addWidget(toolOps, 0, 4);
    mSymGridLayout->addWidget(toolRelations, 0, 5);
    mSymGridLayout->addWidget(toolMisc, 0, 6);

    // second row
    mSymGridLayout->addWidget(toolGreek, 1, 0);
    mSymGridLayout->addWidget(toolGreekBig, 1, 1);
    mSymGridLayout->addWidget(toolSums, 1, 2);
    mSymGridLayout->addWidget(toolIntegrals, 1, 3);
    mSymGridLayout->addWidget(toolProd, 1, 4);
    mSymGridLayout->addWidget(toolSets, 1, 5);
    mSymGridLayout->addWidget(toolArrows, 1, 6);

    // add to Symbols ToolBar
    ui->symbolsBar->addWidget(mSymGrid);

}



void MainWindow::addButtonToMathTabs(const QStringList &list, QWidget *tabWidget)
{
    QHBoxLayout * tabLayout = (QHBoxLayout *)tabWidget->layout();

    if(!tabLayout){
        tabLayout = new QHBoxLayout;
    }

    EquationTemplateWidget *eqTemplate = new EquationTemplateWidget(list, tabWidget);

    connect(eqTemplate, SIGNAL(clicked(QString)), editor, SLOT(replace(QString)) );

    tabLayout->addWidget(eqTemplate);
    tabLayout->addStretch(0);

    tabLayout->setSpacing(0);
    tabLayout->setContentsMargins(0,0,0,0);

    tabWidget->setLayout(tabLayout);

}

void MainWindow::setMathPanelTabsActions()
{
    signalMapper = new QSignalMapper(this);

    // Algebra Tab
    QStringList listAlgebra;
    listAlgebra << ":/resources/icons/tabBar/algebra/eq01.png"
            << ":/resources/icons/tabBar/algebra/eq02.png"
            << ":/resources/icons/tabBar/algebra/eq03.png"
            << ":/resources/icons/tabBar/algebra/eq04.png"
            << ":/resources/icons/tabBar/algebra/eq05.png"
            << ":/resources/icons/tabBar/algebra/eq06.png" ;

    addButtonToMathTabs(listAlgebra,templateTab->getPage(0) );


    // Derivs Tab
    QStringList listDerivs;
    listDerivs << ":/resources/icons/tabBar/derivs/eq01.png"
            << ":/resources/icons/tabBar/derivs/eq02.png"
            << ":/resources/icons/tabBar/derivs/eq03.png"
            << ":/resources/icons/tabBar/derivs/eq04.png"
            << ":/resources/icons/tabBar/derivs/eq05.png"
            << ":/resources/icons/tabBar/derivs/eq06.png" ;

    addButtonToMathTabs(listDerivs, templateTab->getPage(1));

    // Stats Tab
    QStringList listStats;
    listStats << ":/resources/icons/tabBar/stats/eq01.png"
            << ":/resources/icons/tabBar/stats/eq02.png"
            << ":/resources/icons/tabBar/stats/eq03.png"
            << ":/resources/icons/tabBar/stats/eq04.png"
            << ":/resources/icons/tabBar/stats/eq05.png"
            << ":/resources/icons/tabBar/stats/eq06.png"
            << ":/resources/icons/tabBar/stats/eq07.png" ;

    addButtonToMathTabs(listStats, templateTab->getPage(2));

    // Matrix Tab
    QStringList listMatrix;
    listMatrix << ":/resources/icons/tabBar/matrix/eq01.png"
            << ":/resources/icons/tabBar/matrix/eq02.png"
            << ":/resources/icons/tabBar/matrix/eq03.png"
            << ":/resources/icons/tabBar/matrix/eq04.png";

    addButtonToMathTabs(listMatrix, templateTab->getPage(3));

    // Sets Tab
    QStringList listSets;
    listSets << ":/resources/icons/tabBar/sets/eq01.png"
            << ":/resources/icons/tabBar/sets/eq02.png";


    addButtonToMathTabs(listSets, templateTab->getPage(4));

    // Trig Tab
    QStringList listTrig;
    listTrig << ":/resources/icons/tabBar/trig/eq01.png"
            << ":/resources/icons/tabBar/trig/eq02.png"
            << ":/resources/icons/tabBar/trig/eq03.png"
            << ":/resources/icons/tabBar/trig/eq04.png";

    addButtonToMathTabs(listTrig, templateTab->getPage(5));

    // Geometry Tab
    QStringList listGeometry;
    listGeometry << ":/resources/icons/tabBar/geometry/eq01.png"
            << ":/resources/icons/tabBar/geometry/eq02.png"
            << ":/resources/icons/tabBar/geometry/eq03.png"
            << ":/resources/icons/tabBar/geometry/eq04.png";

    addButtonToMathTabs(listGeometry, templateTab->getPage(6));

    // Chemistry Tab
    QStringList listChemistry;
    listChemistry << ":/resources/icons/tabBar/chemistry/eq01.png"
            << ":/resources/icons/tabBar/chemistry/eq02.png"
            << ":/resources/icons/tabBar/chemistry/eq03.png"
            << ":/resources/icons/tabBar/chemistry/eq04.png"
            << ":/resources/icons/tabBar/chemistry/eq05.png";

    addButtonToMathTabs(listChemistry, templateTab->getPage(7));

    // Physics Tab
    QStringList listPhysics;
    listPhysics << ":/resources/icons/tabBar/physics/eq01.png"
            << ":/resources/icons/tabBar/physics/eq02.png"
            << ":/resources/icons/tabBar/physics/eq03.png"
            << ":/resources/icons/tabBar/physics/eq04.png";

    addButtonToMathTabs(listPhysics, templateTab->getPage(8));


    ui->templatesBar->addWidget(templateTab);

}

void MainWindow::setSignals()
{
    /* MainWindow */
    // Menu Bar
    // File Menu
    connect(ui->actionNew, SIGNAL(triggered()), this, SLOT(newEquation()) );
    connect(ui->actionOpen, SIGNAL(triggered()), this, SLOT(open()) );
    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(save()) );
    connect(ui->actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()) );
    connect(ui->actionQuit, SIGNAL(triggered()), this, SLOT(close()) );
    // Export To...
    connect(ui->actionExport_DVI, SIGNAL(triggered()), this, SLOT(exportToDVI()) );
    connect(ui->actionExport_GIF, SIGNAL(triggered()), this, SLOT(exportToGIF()) );
    connect(ui->actionExport_PNG, SIGNAL(triggered()), this, SLOT(saveAs()) );
    connect(ui->actionExport_TeX, SIGNAL(triggered()), this, SLOT(exportToTeX()) );
    connect(ui->actionExport_PDF, SIGNAL(triggered()), this, SLOT(exportToPDF()) );
    connect(ui->actionExport_PS, SIGNAL(triggered()), this, SLOT(exportToPS()) );
    connect(ui->actionExport_ROOT, SIGNAL(triggered()), this, SLOT(exportToROOT()) );

    // Edit Menu
    connect(ui->actionEdit_Preferences, SIGNAL(triggered()), this, SLOT(onPreferences()) );
    connect(ui->actionEdit_Delete, SIGNAL(triggered()), this, SLOT(onDeleteSelection()) );
    // View Menu
    connect(ui->actionView_Refresh, SIGNAL(triggered()), this, SLOT(updateEquation()) );

    // Search Menu
    connect(ui->actionFind, SIGNAL(triggered()), this, SLOT(showFindDialog()));
    connect(ui->actionFind_Next, SIGNAL(triggered()), editor, SLOT(findNext()) );
    connect(ui->actionFind_Previous, SIGNAL(triggered()), editor, SLOT(findPrevious()) );
    connect(ui->actionReplace, SIGNAL(triggered()), this, SLOT(showReplaceDialog()) );
    connect(findWidget, SIGNAL(closing()), editor, SLOT(clearSelections()) );
    connect(replaceDialog, SIGNAL(closing()), editor, SLOT(clearSelections()) );

    // Window Menu

    // Help Menu
    connect(ui->actionHelp_About, SIGNAL(triggered()), this, SLOT(onAbout()) );
    connect(ui->actionHelp_AboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()) );

    // GUI
    connect(editor, SIGNAL(textChanged()), this, SLOT(editorChanged()) );
    connect(editor, SIGNAL(selectionChanged()), this, SLOT(editorChanged()) );

    connect(ui->editPreamble, SIGNAL(textChanged()), this, SLOT(editorChanged()) );
    connect(ui->fontSizeComboBox, SIGNAL(currentIndexChanged(int)) , this, SLOT(onEquationChanged()) );
    connect(ui->fgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(onEquationChanged()) );
    connect(ui->bgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(onEquationChanged()) );
    connect(ui->inlineCheckBox, SIGNAL(toggled(bool)), this, SLOT(onEquationChanged()) );

    connect(preferencesDialog, SIGNAL(accepted()), this, SLOT(writeSettings()) );

    connect(ui->renderButton, SIGNAL(clicked()), this, SLOT(updateEquation()) );
    // Find widget
    connect(findWidget, SIGNAL(changedFindText(QString)), this, SLOT(onFindTextChanged(QString)) );
    connect(findWidget, SIGNAL(find(QString,QTextDocument::FindFlags)), editor, SLOT(find(QString,QTextDocument::FindFlags)) );
    connect(findWidget, SIGNAL(findAll(QString,QTextDocument::FindFlags)), editor, SLOT(findAll(QString,QTextDocument::FindFlags)) );

    // Replace Dialog
    connect(replaceDialog, SIGNAL(find(QString,QTextDocument::FindFlags)), editor, SLOT(findAll(QString,QTextDocument::FindFlags)) );
    connect(replaceDialog, SIGNAL(replace(QString)), editor, SLOT(replace(QString)) );
    connect(replaceDialog, SIGNAL(replaceAll(QString)), editor, SLOT(replaceAll(QString)) );
    // Misc
    connect(this, SIGNAL(equationChanged(bool)), this, SLOT(onEquationChanged(bool)) );
    connect(this, SIGNAL(zoomChanged()), this, SLOT(onZoomChanged()) );


    //for Symbols Toolbar
    connect(toolDeco, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolFunc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolDelim, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolAccents, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolOps, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolRelations, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolMisc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSets, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreek, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreekBig, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSums, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolIntegrals, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolProd, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolArrows, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );

    connect(preferencesDialog->renderUpdateManualRadio, SIGNAL(toggled(bool)), ui->renderWidget, SLOT(setVisible(bool)) );

}


void MainWindow::closeEvent(QCloseEvent *event)
{
    if ( wishToSave() ) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

bool MainWindow::exportToDVI()
{
    QString outFilename = saveTo("DVI File (*.dvi)");

    if(outFilename.isEmpty())
        return false;

    QString tempFile = QDir::toNativeSeparators( QDir::tempPath() + QString("/%1.dvi").arg(TEMP_FILE_NAME) );

    QString mess = "Copy " + tempFile + " -> " + outFilename;

    QFile::remove(outFilename);

    bool status = QFile::copy(tempFile, outFilename);

    if(!status)
        QMessageBox::critical(this, "Error", mess );
	 return status;
}

bool MainWindow::exportToGIF()
{
    QString outExt(".gif");
    QString inExt(".dvi");

    QString outFilename = saveTo("GIF Image (*.gif)");
    QString inFilename  = Util::getTempFile().append(inExt);

    if(outFilename.isEmpty())
        return false;


    outFilename = "\""+outFilename+"\"";
    int fontResolution = Util::getFontResolution( ui->fontSizeComboBox->currentText().toInt() );

    QString appComm = QString("\"%1\" %2").arg(preferencesDialog->editGIF->text().trimmed()).arg(preferencesDialog->editArgsGIF->text().trimmed());
    QString fullComm = QDir::toNativeSeparators( appComm.arg(outFilename).arg(inFilename).arg(fontResolution).arg(ui->bgColorPicker->getColorString()).arg(zoomFactor) );

    return runCommand(fullComm);
}

bool MainWindow::exportToPDF()
{
    QString outExt(".pdf");
    QString inExt(".dvi");

    QString outFilename = saveTo("PDF File ( *.pdf )");
    QString inFilename  = Util::getTempFile().append(inExt);

    if(outFilename.isEmpty())
        return false;

    outFilename = "\""+outFilename+"\"";

    QString appComm = QString("\"%1\" %2").arg(preferencesDialog->editPDF->text().trimmed()).arg(preferencesDialog->editArgsPDF->text().trimmed());
    QString fullComm = QDir::toNativeSeparators( QString(appComm).arg(outFilename).arg(inFilename) );

    return runCommand(fullComm);
}

bool MainWindow::exportToPS()
{
    QString outExt(".ps");
    QString inExt(".dvi");

    QString outFilename = saveTo(" Postscript File ( *.ps ) ");
    QString inFilename  = Util::getTempFile().append(inExt);

    if(outFilename.isEmpty())
        return false;

    outFilename = "\""+outFilename+"\"";

    QString appComm = QString("\"%1\" %2").arg(preferencesDialog->editPS->text().trimmed()).arg(preferencesDialog->editArgsPS->text().trimmed());
    QString fullComm = QDir::toNativeSeparators( QString(appComm).arg(outFilename).arg(inFilename) );

    return runCommand(fullComm);

}

bool MainWindow::exportToTeX()
{
    QString outFilename = saveTo("TeX File (*.tex)");

    if(outFilename.isEmpty())
        return false;

    QString tempFile = QDir::toNativeSeparators( QDir::tempPath() + QString("/%1.tex").arg(TEMP_FILE_NAME) );

    QString mess = "Copy " + tempFile + " -> " + outFilename;

    QFile::remove(outFilename);

    bool status = QFile::copy(tempFile, outFilename);

    if(!status)
        QMessageBox::critical(this, "Error", mess );
	 return status;
}

bool MainWindow::exportToROOT()
{
    QString outFilename = saveTo("ROOT Macro File (*.C)");

    if(outFilename.isEmpty())
        return false;

    QFile saveFile(outFilename);

    if (!saveFile.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;


    QString rootEquation = editor->toPlainText();

    QString rootContent= QString("TLatex %1(0,0, \"").arg(TEMP_FILE_NAME);


    QTextStream out(&saveFile);
    out << rootContent;
    out.flush();

    QChar c;
    for (int i=0; i<rootEquation.size(); i++){
        c = rootEquation.at(i);

        if( c == QChar('\\') )
            c = QChar('#');

        out << c;
    }
    out.flush();

    out << "\");";

    saveFile.close();


    return true;
}

void MainWindow::loadTexTemplate()
{
    QFile file(":/resources/templates/template-content.tex");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    texTemplateContent = in.readAll();

    QFile file2(":/resources/templates/template-preamble.tex");
    if (!file2.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in2(&file2);
    QString texTemplatePreamble = in2.readAll();

    ui->editPreamble->setText(texTemplatePreamble);

}

void MainWindow::newEquation()
{
    if (wishToSave()) {
        editor->clear();
        setCurrentFile("");
    }

}

void MainWindow::open()
{
    QString curDirToSave;

    if (wishToSave()) {

        if( curFile.isEmpty() )
            curDirToSave = QDir::homePath();
        else
            curDirToSave = QFileInfo(curFile).dir().path();

        QString fileName = QFileDialog::getOpenFileName(this, tr("Open Equation"),
                                                        curDirToSave,
                                                        tr("Image Equation (*.png)"));
        if (!fileName.isEmpty())
            loadEquation(fileName);

    }
}

void MainWindow::openRecentFile()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action){
        loadEquation(action->data().toString());
    }
}

bool MainWindow::save()
{
    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        return saveEquation(curFile);
    }
}

bool MainWindow::saveAs()
{
    QString fileName = saveTo("Equation Image (*.png)");

    if (fileName.isEmpty())
        return false;

    return saveEquation(fileName);
}

bool MainWindow::saveEquation(QString &fileName)
{
    if(!fileName.endsWith(".png", Qt::CaseInsensitive) ){
        fileName.append(".png");
    }

    equation->pixmap().save(fileName, "PNG");
    writePNGMetaData(fileName);

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("File saved"), 5000);

    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);
    QStringList files = settings.value("RecentFileList").toStringList();
    files.removeAll(fileName);
    files.prepend(fileName);
    while (files.size() > MAX_RECENT_FILES)
        files.removeLast();

    settings.setValue("RecentFileList", files);

    updateRecentFileActions();

    return true;
}


QString MainWindow::saveTo(const QString &filter)
{
    QString curDirToSave;

    if( curFile.isEmpty() )
        curDirToSave = QDir::homePath();
    else
        curDirToSave = QFileInfo(curFile).dir().path();

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Equation as..."), curDirToSave, filter );

    return fileName;
}

void MainWindow::onDeleteSelection()
{
    editor->textCursor().removeSelectedText();
}

bool MainWindow::onPreferences()
{
    preferencesDialog->show();

    return true;
}

void MainWindow::onAbout()
{

    QFile file(":/LICENSE");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    QString licenseText = in.readAll();
    in.flush();


    aboutDialog->setLicenseText(licenseText);
    aboutDialog->show();

}

void MainWindow::writePNGMetaData(const QString &pngFile)
{
    // editor->selectAll();
    // ui->editPreamble->selectAll();

    //QTextDocumentFragment equation = editor->textCursor().selection();
    //QTextDocumentFragment preamble = ui->editPreamble->textCursor().selection();


    QString latexEqString = editor->toPlainText().replace('\n', "[--break--]");
    QString latexEqPreamble = ui->editPreamble->toPlainText().replace('\n', "[--break--]");

    QImage tmp_imgFile(pngFile);
    if(! tmp_imgFile.isNull()){
        tmp_imgFile.setText(INFO_HEADER_GEN, APP_NAME);
        tmp_imgFile.setText(INFO_HEADER_GEN_VER,APP_VERSION );
        tmp_imgFile.setText(INFO_HEADER_LATEX, latexEqString ) ;
        tmp_imgFile.setText(INFO_HEADER_FG, ui->fgColorPicker->getColorName() );
        tmp_imgFile.setText(INFO_HEADER_BG, ui->bgColorPicker->getColorName() );
        tmp_imgFile.setText(INFO_HEADER_RES, QString::number(ui->fontSizeComboBox->currentIndex()) );
        tmp_imgFile.setText(INFO_HEADER_PREAMBLE, latexEqPreamble );
        tmp_imgFile.save(pngFile);
    }

    editor->textCursor().clearSelection();
    ui->editPreamble->textCursor().clearSelection();

}

void MainWindow::readSettings()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("GENERAL");
    preferencesDialog->recentEqNumber->setValue(settings.value("max_recent", DEFAULT_MAX_RECENT_FILES ).toInt() );
    preferencesDialog->checkRememberLayout->setChecked( settings.value("remember_layout", DEFAULT_REMEMBER_LAYOUT).toBool() );

    if( preferencesDialog->checkRememberLayout->isChecked() ){
        ui->actionStandard_Bar->setChecked( settings.value("mainbar", DEFAULT_SHOW_MAIN_TOOLBAR).toBool() );
        ui->actionSymbols_Bar->setChecked( settings.value("panel_symbols", DEFAULT_SHOW_PANEL_SYMBOLS).toBool() );
        ui->actionTemplate_Bar->setChecked( settings.value("panel_templates", DEFAULT_SHOW_PANEL_TEMPLATES).toBool() );
        ui->actionProperties_Bar->setChecked(settings.value("panel_properties", DEFAULT_SHOW_PANEL_PROPERTIES).toBool() );

        QPoint pos = settings.value("position", QPoint(DEFAULT_WINDOW_X, DEFAULT_WINDOW_Y)).toPoint();
        QSize size = settings.value("size", QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT)).toSize();
        resize(size);
        move(pos);
    }

    preferencesDialog->comboExportType->setCurrentIndex(settings.value("default_export", DEFAULT_EXPORT_TYPE ).toInt());
    settings.endGroup();

    settings.beginGroup("EDITOR");
    preferencesDialog->comboFontFamily->setCurrentIndex(settings.value("font", DEFAULT_FONT).toInt() );
    preferencesDialog->comboFontSize->setCurrentIndex( settings.value("font_size", DEFAULT_FONT_SIZE).toInt() );
    preferencesDialog->checkSyntaxHighligthing->setChecked(settings.value("highlighting", DEFAULT_HIGHLIGHTING ).toBool());
    preferencesDialog->checkTextWrapping->setChecked( settings.value("textwrap", DEFAULT_WRAPPING).toBool() );
    preferencesDialog->groupBoxCompletion->setChecked( settings.value("completion", DEFAULT_COMPLETION).toBool() );
    preferencesDialog->checkSensitiveCompletion->setChecked( settings.value("case_sensitive_completion", DEFAULT_COMPLETION).toBool() );
    settings.endGroup();

    settings.beginGroup("PREVIEW");
    preferencesDialog->renderUpdateAutoRadio->setChecked( settings.value("updateAuto", DEFAULT_UPDATE_AUTO ).toBool() );
    preferencesDialog->renderUpdateManualRadio->setChecked(! settings.value("updateAuto", DEFAULT_UPDATE_AUTO ).toBool() );

    preferencesDialog->spinUpdateTime->setValue(settings.value("updateTime", DEFAULT_UPDATE_TIME ).toInt());


    settings.endGroup();

    settings.beginGroup("LATEX");
    preferencesDialog->editDVI->setText(settings.value("latexdvi", DEFAULT_LATEXDVI ).toString());
    preferencesDialog->editArgsDVI->setText(settings.value("latexdvi_args", DEFAULT_LATEXDVI_ARGS).toString());
    preferencesDialog->editPNG->setText(settings.value("dvipng", DEFAULT_DVIPNG ).toString());
    preferencesDialog->editArgsPNG->setText(settings.value("dvipng_args", DEFAULT_DVIPNG_ARGS).toString());
    preferencesDialog->editPDF->setText(settings.value("dvipdf", DEFAULT_DVIPDF ).toString());
    preferencesDialog->editArgsPDF->setText(settings.value("dvipdf_args", DEFAULT_DVIPDF_ARGS).toString());
    preferencesDialog->editGIF->setText(settings.value("dvigif", DEFAULT_DVIGIF ).toString());
    preferencesDialog->editArgsGIF->setText(settings.value("dvigif_args", DEFAULT_DVIGIF_ARGS).toString());
    preferencesDialog->editPS->setText(settings.value("dvips", DEFAULT_DVIPS ).toString());
    preferencesDialog->editArgsPS->setText(settings.value("dvips_args", DEFAULT_DVIPS_ARGS).toString());
    settings.endGroup();

    updateUI();
}

bool MainWindow::runCommand(const QString &commandStr)
{
    qDebug() << "\n\n ============= [Process] ==========" << commandStr << " ====== ";

    QProcess process(this);
    process.setProcessChannelMode(QProcess::MergedChannels);
    process.setWorkingDirectory(QDir::tempPath());

    process.start(commandStr);

    if (!process.waitForFinished()) {
        qDebug() << "[Failed] " << process.errorString();
		  return false;
	 }
    else
        qDebug() << "[Output]" << process.readAll();
	 return true;
}

void MainWindow::setupParser()
{
    parser.addToken("font");
    parser.addToken("foreground_color");
    parser.addToken("background_color");
    parser.addToken("displaymath", "show");
    parser.addToken("input");
}

void MainWindow::updateDisplay()
{

    equation->setPixmap( QDir::toNativeSeparators(QDir::tempPath()+"/"+Util::getTempFile().append(".png")) );

    QRectF imageBorders = equation->boundingRect();

    scene->setSceneRect(imageBorders);

    equation->setPos(scene->width()/2 - imageBorders.width()/2, scene->height()/2 - imageBorders.height()/2);

}

void MainWindow::updateImage()
{
    int fontResolution = Util::getFontResolution( ui->fontSizeComboBox->currentText().toInt() );


    QString dvi2png = "\""+preferencesDialog->editPNG->text().trimmed()+"\""+" "+preferencesDialog->editArgsPNG->text().trimmed();

    QString cmdStrDVI2PNG = QDir::toNativeSeparators( QString(dvi2png).arg( Util::getTempFile().append(".png") ).arg( Util::getTempFile().append(".dvi")).arg(fontResolution).arg(ui->bgColorPicker->getColorString()).arg(zoomFactor) );


    qDebug() << "\n\n[process]" << cmdStrDVI2PNG;

    QProcess process2(this);
    process2.setProcessChannelMode(QProcess::MergedChannels);
    process2.setWorkingDirectory(QDir::tempPath());

    process2.start(cmdStrDVI2PNG);

    if (!process2.waitForFinished())
        qDebug() << "[Failed] " << process2.errorString();
    else
        qDebug() << "[output]" << process2.readAll();

    QString imgFileStr = QDir::toNativeSeparators(QDir::tempPath()+"/"+Util::getTempFile().append(".png"));

    qDebug() << "METADATA FILE =" << imgFileStr;

    writePNGMetaData(imgFileStr);


}

void MainWindow::updateEquation()
{
    qDebug() << "calling Update ";

    if(isDirty){
        qDebug() << "\t\t\t ========== Updateing ============= ";

        QString latexEquation = editor->toPlainText();
        QString texTemplatePreamble = ui->editPreamble->toPlainText();

        QString fgColor = ui->fgColorPicker->getColorString();
        QString bgColor = ui->bgColorPicker->getColorString();


        // make highlighted text visible in Preview
        QString insert1 = QString("\\colorbox{fgC}{\\color{bgC} $");
        QString insert2("$}");

        int insertStr1Len = insert1.length();
        int insertStr2Len = insert2.length();
        int insertStrLen = insertStr1Len + insertStr2Len;
        int newInsert1Pos, newInsert2Pos;

        HLSelections selections = editor->getSelections();

        for(int i=0; i < selections.size(); i++){
            SelectionIndex *selectionItem = selections.at(i);

            int ss = selectionItem->start; // start selection
            int se = selectionItem->end; // end selection

            newInsert1Pos = ss + i*insertStrLen;
            newInsert2Pos = se + i*insertStrLen + insertStr1Len;


            latexEquation.insert(newInsert1Pos, insert1);
            latexEquation.insert(newInsert2Pos, insert2);

        }

        qDebug() << "EQUATION With HIGHLIGHTING " << latexEquation;

        // parse Tex template
        parser.setTokenValue("font", "sans");
        parser.setTokenValue("foreground_color", fgColor);
        parser.setTokenValue("background_color", bgColor);

        if( ui->inlineCheckBox->isChecked() )
        {
            parser.setTokenValue("displaymath", "");
            latexEquation = "$"+latexEquation+"$";
        }
        else{
            parser.setTokenValue("displaymath", "show");
        }

        parser.setTokenValue("input", latexEquation);

        QString parsedTexContent = parser.parse(texTemplatePreamble+texTemplateContent);

        QString texFile = QDir::toNativeSeparators(QDir::tempPath()+"/"+Util::getTempFile().append(".tex"));
        qDebug() << "tex File is " << texFile;
        // write parsed Tex content to a temp file
        QFile tmpTexFile( texFile  );

        if (!tmpTexFile.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&tmpTexFile);
        out << parsedTexContent;
        out.flush();
        tmpTexFile.close();

        QString latex2dvi = "\""+preferencesDialog->editDVI->text().trimmed()+"\""+" "+preferencesDialog->editArgsDVI->text().trimmed();
        // create commands
        QString cmdStrTex2DVI = QDir::toNativeSeparators( QString(latex2dvi).arg(Util::getTempFile().append(".tex")) );


        qDebug() << "\n\n[process]" << cmdStrTex2DVI;

        QProcess process1(this);

        process1.setProcessChannelMode(QProcess::MergedChannels);
        process1.setWorkingDirectory(QDir::tempPath());

        process1.start(cmdStrTex2DVI);

        if (!process1.waitForFinished())
            qDebug() << "[Failed] " << process1.errorString();
        else
            qDebug() << "[output]" << process1.readAll();

        updateImage();
        updateDisplay();

        equation->setVisible(true);

        isDirty = false;
        nUpdates++;

    }
}

void MainWindow::updateUI()
{
    editor->selectAll();
    editor->setHighLighting( preferencesDialog->checkSyntaxHighligthing->isChecked() );
    editor->setCompletion( preferencesDialog->groupBoxCompletion->isChecked() );
    editor->setCompletionSensitive( preferencesDialog->checkSensitiveCompletion->isChecked() );
    editor->setFontFamily( preferencesDialog->comboFontFamily->currentText() );
    editor->setFontPointSize( preferencesDialog->comboFontSize->currentText().toInt() );

    if( preferencesDialog->checkTextWrapping->isChecked() )
        editor->setLineWrapMode(QTextEdit::WidgetWidth);
    else
        editor->setLineWrapMode(QTextEdit::NoWrap);

    renderAuto = preferencesDialog->renderUpdateAutoRadio->isChecked();
    ui->renderWidget->setVisible( !renderAuto );
}

void MainWindow::updateRecentFileActions()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);
    QStringList files = settings.value("RecentFileList").toStringList();

    if(files.isEmpty() || files.size() == 0 )
        return ;

    int numRecentFiles = qMin( preferencesDialog->recentEqNumber->value(), files.size() );
    for (int i = 0; i < numRecentFiles; ++i) {
           QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
           recentFileActions[i]->setText(text);
           recentFileActions[i]->setData(files[i]);
           recentFileActions[i]->setVisible(true);
       }

    for (int j = numRecentFiles; j < MAX_RECENT_FILES; ++j)
        recentFileActions[j]->setVisible(false);

}

void MainWindow::writeSettings()
{

    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("GENERAL");
    settings.setValue("max_recent", preferencesDialog->recentEqNumber->value() );
    settings.setValue("remember_layout", preferencesDialog->checkRememberLayout->isChecked() );
    settings.setValue("position", pos() );
    settings.setValue("size", size());
    settings.setValue("mainbar", ui->mainBar->isVisible() );
    settings.setValue("panel_properties", ui->propertiesBar->isVisible() );
    settings.setValue("panel_symbols", ui->symbolsBar->isVisible() );
    settings.setValue("panel_templates", ui->templatesBar->isVisible() );
    settings.setValue("default_export", preferencesDialog->comboExportType->currentIndex() );
    settings.endGroup();

    settings.beginGroup("EDITOR");
    settings.setValue("font", preferencesDialog->comboFontFamily->currentIndex() );
    settings.setValue("font_size", preferencesDialog->comboFontSize->currentIndex() );
    settings.setValue("highlighting", preferencesDialog->checkSyntaxHighligthing->isChecked() );
    settings.setValue("textwrap", preferencesDialog->checkTextWrapping->isChecked() );
    settings.setValue("completion", preferencesDialog->groupBoxCompletion->isChecked() );
    settings.setValue("case_sensitive_completion", preferencesDialog->checkSensitiveCompletion->isChecked() );
    settings.endGroup();

    settings.beginGroup("PREVIEW");
    settings.setValue("updateAuto", preferencesDialog->renderUpdateAutoRadio->isChecked() );
    settings.setValue("updateTime", preferencesDialog->spinUpdateTime->value() );
    settings.endGroup();

    settings.beginGroup("LATEX");
    settings.setValue("latexdvi", preferencesDialog->editDVI->text() );
    settings.setValue("latexdvi_args", preferencesDialog->editArgsDVI->text() );
    settings.setValue("dvipng", preferencesDialog->editPNG->text() );
    settings.setValue("dvipng_args", preferencesDialog->editArgsPNG->text() );
    settings.setValue("dvipdf", preferencesDialog->editPDF->text() );
    settings.setValue("dvipdf_args", preferencesDialog->editArgsPDF->text() );
    settings.setValue("dvips", preferencesDialog->editPS->text() );
    settings.setValue("dvips_args", preferencesDialog->editArgsPS->text() );
    settings.setValue("dvigif", preferencesDialog->editGIF->text() );
    settings.setValue("dvigif_args", preferencesDialog->editArgsGIF->text() );
    settings.endGroup();

    updateUI();
    updateRecentFileActions();
}

void MainWindow::editorChanged()
{
    setWindowModified(editor->document()->isModified());

    emit equationChanged(true);
}

void MainWindow::onEquationChanged(bool redraw)
{
    isDirty = redraw;

    int startTimer = preferencesDialog->spinUpdateTime->value();


    statusBar()->showMessage(tr("Equation changed. Updating in %1 ms").arg(startTimer), startTimer);

    if(renderAuto)
        QTimer::singleShot(startTimer, this, SLOT(updateEquation()) );

}

void MainWindow::onFindTextChanged(const QString &exp)
{
    // if we have a expression to search
    bool findable = (! exp.isEmpty()) && !exp.isNull();

    // Enable/Disable widgets from Search Menu accordingly
    ui->actionFind_Next->setEnabled(findable);
    ui->actionFind_Previous->setEnabled(findable);

    editor->setTextCursor( QTextCursor(editor->document()) );

}

void MainWindow::showFindDialog()
{
    findWidget->setFindExpr( editor->selectedText() );
    findWidget->show();

    editor->setTextCursor( QTextCursor(editor->document()) );
}

void MainWindow::showReplaceDialog()
{
    replaceDialog->setFindExpr( editor->selectedText() );
    replaceDialog->show();

    editor->setTextCursor( QTextCursor(editor->document()) );

}

void MainWindow::insertSymbol(QAction *action)
{
    editor->insertPlainText(action->text() );
}

bool MainWindow::wishToSave()
{
    if ( isWindowModified () ) {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, APP_FULL_NAME,
                                   tr("The Equation has been modified.\n"
                                      "Do you want to save your changes?"),
                                   QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }

    return true;
}

void MainWindow::loadEquation(const QString &fileName)
{
    qDebug() << "Loading equation " << fileName;
    QImage m_image(fileName, "PNG");

    if(m_image.isNull()){
        QMessageBox::critical(this, APP_FULL_NAME+QString(" - File not found!"),
                              QString("Can not load file\n %1").arg(fileName),
                              QMessageBox::Ok);

        return;
    }


    // check if file was created using EqualX
    if( !(m_image.text(INFO_HEADER_GEN) == APP_NAME) ){
        QMessageBox::StandardButton a = QMessageBox::warning(this, APP_FULL_NAME,
                                                             QString("This file was not created using %1.\n Do you want to try to load it anyway?").arg(APP_FULL_NAME),
                                                             QMessageBox::Ok | QMessageBox::Cancel);

        if( a == QMessageBox::Cancel ) return;


    }


    // check if file has latex information
    QString latexString = m_image.text(INFO_HEADER_LATEX).replace("[--break--]", QChar('\n'));
    if(latexString.isEmpty() ){
        QMessageBox::information(this, APP_FULL_NAME,
                                 QString("This equation does not contain any latex information.\n %1 can not load it.").arg(APP_FULL_NAME),
                                 QMessageBox::Ok );

        statusBar()->showMessage(tr("File not loaded"), 2000);
        return;

    }

    QString latexEqPreamble =  m_image.text(INFO_HEADER_PREAMBLE).replace("[--break--]", QChar('\n'));

    QTextDocumentFragment equation = QTextDocumentFragment::fromPlainText(latexString);
    QTextDocumentFragment preamble = QTextDocumentFragment::fromPlainText(latexEqPreamble);


    // set widgets data
    editor->clear();
    editor->textCursor().insertFragment(equation );

    ui->editPreamble->clear();
    ui->editPreamble->textCursor().insertFragment(preamble);

    ui->fgColorPicker->setColorName(m_image.text(INFO_HEADER_FG) );
    ui->bgColorPicker->setColorName(m_image.text(INFO_HEADER_BG) );
    ui->fontSizeComboBox->setCurrentIndex( m_image.text(INFO_HEADER_RES).toInt() );


    setCurrentFile(fileName);

    editor->textCursor().clearSelection();
    ui->editPreamble->textCursor().clearSelection();

    statusBar()->showMessage( QString("File loaded: %1").arg(fileName), 5000);
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    setWindowFilePath(curFile);

    editor->document()->setModified(false);
    setWindowModified(false);

    QString shownName;
    if (curFile.isEmpty())
        shownName = "untitled.png";
    else
        shownName = strippedName(curFile);

    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(APP_FULL_NAME));




}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}
